#!/usr/bin/env python
# -*- coding: utf-8 -*-

#    Copyright (C) <2009,2011>  <Sebastian Kacprzak> <naicik |at| gmail |dot| com>

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from AbstractPycurlPlugin import AbstractPycurlPlugin
from pycurl import FORM_FILE
#from helpers.gnomeHelper import getMimeTypes
from StringIO import StringIO
from xml.dom import minidom
from libs.uploadHelper import createLinks
from libs.cliHelper import printLink, printProgressToCLI

class Imageshack_us(AbstractPycurlPlugin):
    NAME = "Imageshack"
    acceptedFiletypes = ["jpg", "bmp", "gif", "png", "tif"]
    canUsePassword = True

    def __init__(self, fileNumber, progressCallbackFunction):
        self.fileNumber = fileNumber
        self.progressCallbackFunction = progressCallbackFunction

    def parseResponse(self, xml, fileName = None):
        """parse given xml
        returns dictionary with keys: IM, Forum, Alt Forum, HTML, Direct, Forum Thumb, Alt Forum Thumb, HTML Thumb, Twitter Link"""
        xmldoc = minidom.parse(StringIO(xml))

        imageLink = xmldoc.getElementsByTagName("image_link")[0].firstChild.data #xmldoc.childNodes[0].childNodes[1].firstChild.data
        thumbLink = xmldoc.getElementsByTagName("thumb_link")[0].firstChild.data #print xmldoc.childNodes[0].childNodes[3].firstChild.data
        adLink = xmldoc.getElementsByTagName("ad_link")[0].firstChild.data #print xmldoc.childNodes[0].childNodes[5].firstChild.data
        thumbExists = xmldoc.getElementsByTagName("thumb_exists")[0].firstChild.data #print xmldoc.childNodes[0].childNodes[7].firstChild.data
        if thumbExists == "no":
            thumbLink = imageLink # thumb was not generated, use image as a thumb
        links = createLinks(adLink, None, imageLink,thumbLink)
        return links#, [imageLink, thumbLink, adLink]

    def progress(self, download_t, download_d, upload_t, upload_d):
        """passes self.fileNumber and arguments to self.progressCallbackFunction"""
        self.progressCallbackFunction(self.fileNumber, download_t, download_d, upload_t, upload_d)

    def __changePngExtensionIfNeeded(self, file):
        #workaround for imageshack failing to accept png https://bugs.launchpad.net/pyshare/+bug/587255
        #copies png files to temporary file with jpg extension
        #imports are embed to make it easier to delete workaround when no longer needed
        if file.split('.')[-1].lower() == "png":
            from tempfile import mktemp
            tempName = mktemp(".jpg")
            from shutil import copyfile
            copyfile(file, tempName)
            file = tempName
        return file
    
    def upload(self, file, callbackFunction, username, password):
        """Uploads file, and passed links to callbackFunction.
        Optionaly this method may call self.progress to inform about upload status"""

        file = self.__changePngExtensionIfNeeded(file)
        post =  [('fileupload', (FORM_FILE, file)),  ('xml', 'yes')]
        if username:
            post.append(('a_username', username))
            post.append(('a_password', password))
        self.uploadUsingPycurlPost(file, callbackFunction,
            "http://www.imageshack.us/index.php",
            #"http://www.imageshack.us/upload_api.php", #returns more info, that we don't need
            post)



#used only if __name__ == "__main__"
from threading import Thread
from threading import BoundedSemaphore
def uploadFile(file, resultCallbackFunction=printLink, progressCallbackFunction=printProgressToCLI, fileNumber = 0, semaphore = BoundedSemaphore(2), username="", password=""):
    sti = Imageshack_us(fileNumber, progressCallbackFunction)
    sti.upload(file, semaphore, resultCallbackFunction)

#used only if __name__ == "__main__"
def uploadFiles(files, resultCallbackFunction=printLink, progressCallbackFunction=printProgressToCLI, fileNumber = 0, semaphore = BoundedSemaphore(2)):
    """uploads given files in threads
    This function is non blocking, you should give a callback function as parameter if you want to know when it ended or what is the progress
    Error will be thrown if upload was unsuccessful"""
    threadList = []
    for file in files:
        sti = Imageshack_us(fileNumber, progressCallbackFunction)
        fileNumber += 1
        t = Thread(None, sti.upload, None,(file, semaphore, resultCallbackFunction))
        t.start()
        threadList.append(t)
    return threadList


if __name__ == "__main__":
    from sys import argv
    if len(argv) > 1:
        args = argv[1:] # first argument is file name, so ignore it
        threadList = uploadFiles(args)
        for thread in threadList:
            thread.join() # wait for all threads to end
